"""
Module to get delays between different nodes
:author: gurnoorsingh(20221110)
"""

import re
import os
import json
import subprocess
from constants import RELAYS
from tqdm import tqdm
from collections import defaultdict
from txn_to_sender_map import get_txn_receiver_stats

def get_sender_relay():
    log_file_path = "/home/azureuser/.algorand/node.log"
    command = f"grep OutgoingPeers {log_file_path} | tail -n 1"
    with subprocess.Popen(command, stdout=subprocess.PIPE, stderr=None, shell=True) as process:
        output = process.communicate()[0].decode("utf-8")
        endpoint = json.loads(output)["details"]["OutgoingPeers"][0]["Endpoint"]
    
    return endpoint


def send_txn_and_get_txn_id():
    command = "bash /home/azureuser/algorand-topology/scripts/create_txn.sh"
    with subprocess.Popen(command, stdout=subprocess.PIPE, stderr=None, shell=True) as process:
        output = process.communicate()[0].decode("utf-8")
        txn_id = re.findall(".*Signed transaction with txID: ([A-Z0-9]+).*", output)[0]

    return txn_id


def filter_logs_by_txn_id(txn_id):
    log_file_path = "/home/azureuser/node/data/node.log"
    os.system(f"grep {txn_id} {log_file_path} > filtered_logs/data_{txn_id}")


def main():
    COUNT = 10
    sender_relay = get_sender_relay()
    print("sender:", sender_relay)

    delays, delay_observations = dict(), defaultdict(lambda: 0)
    for relay in RELAYS:
        if relay != sender_relay:
            delays[relay] = 0

    for _ in tqdm(range(COUNT)):
        assert get_sender_relay() == sender_relay

        txn_id = send_txn_and_get_txn_id()
        filter_logs_by_txn_id(txn_id)
        print("txn_id:", txn_id)

        receiver_stats = get_txn_receiver_stats(txn_id)
        for relay in receiver_stats.keys():
            delay_observations[relay] += 1
            if relay != sender_relay:
                delays[relay] += receiver_stats[relay] - receiver_stats[sender_relay]

    for relay in RELAYS:
        if relay != sender_relay:
            if delay_observations[relay] != 0:
                delays[relay] /= delay_observations[relay]
        
    sorted_delays = dict(sorted(delays.items(), key=lambda x: x[1]))
    for relay, delay in sorted_delays.items():
        print(relay, delay if delay !=0 else 'N/A')

if __name__ == '__main__':
    main()